import unittest

from lxml import etree

from mdg.sparx_xmi.parse import attr_parse, class_parse, association_parse
from mdg.uml import UMLClass, UMLPackage, Cardinality, UMLAssociationType


class TestXMIAttributeParse(unittest.TestCase):
    def setUp(self):
        self.tree = etree.fromstring("""
                    <xmi:XMI xmi:version="2.1" xmlns:uml="http://schema.omg.org/spec/UML/2.1" xmlns:xmi="http://schema.omg.org/spec/XMI/2.1" xmlns:thecustomprofile="http://www.sparxsystems.com/profiles/thecustomprofile/1.0">
                        <ownedAttribute xmi:type="uml:Property" xmi:id="EAID_7EBDF95C_3A97_4163_A683_3ACC3CF507E4" name="system" visibility="private" isStatic="false" isReadOnly="false" isDerived="false" isOrdered="false" isUnique="true" isDerivedUnion="false">
                            <type xmi:idref="EAJava_int"/>
                        </ownedAttribute>
                        <attribute xmi:idref="EAID_7EBDF95C_3A97_4163_A683_3ACC3CF507E4" name="id" scope="Private">
                            <initial/>
                            <documentation/>
                            <model ea_localid="1213" ea_guid="{47318F66-BBA6-4e9f-9045-83B2E12E25C8}"/>
                            <properties type="int" precision="0" collection="false" length="0" static="0" duplicates="0" changeability="changeable"/>
                            <coords ordered="0" scale="0"/>
                            <containment containment="Not Specified" position="0"/>
                            <stereotype/>
                            <bounds lower="1" upper="1"/>
                            <options/>
                            <style/>
                            <styleex value="volatile=0;"/>
                            <tags/>
                            <xrefs value="$XREFPROP=$XID={EA9F1375-D590-4c2b-8721-DC0B55BFE4A9}$XID;$NAM=CustomProperties$NAM;$TYP=attribute property$TYP;$VIS=Public$VIS;$PAR=0$PAR;$DES=@PROP=@NAME=isID@ENDNAME;@TYPE=Boolean@ENDTYPE;@VALU=1@ENDVALU;@PRMT=@ENDPRMT;@ENDPROP;$DES;$CLT={47318F66-BBA6-4e9f-9045-83B2E12E25C8}$CLT;$SUP=&lt;none&gt;$SUP;$ENDXREF;"/>
                        </attribute>
                    </xmi:XMI>""")
        self.package = UMLPackage("id", "name")
        self.parent = UMLClass(self.package, "name", "id")

    def test_attr_parse(self):
        element = self.tree.find("ownedAttribute")
        self.assertIsNotNone(element)

        attr = attr_parse(self.parent, element, self.tree)
        self.assertIsNotNone(attr)
        self.assertEqual(attr.classification_id, None)


class TestXMIClassParse(unittest.TestCase):
    def setUp(self):
        self.tree = etree.fromstring("""
                    <xmi:XMI xmi:version="2.1" xmlns:uml="http://schema.omg.org/spec/UML/2.1" xmlns:xmi="http://schema.omg.org/spec/XMI/2.1" xmlns:thecustomprofile="http://www.sparxsystems.com/profiles/thecustomprofile/1.0">
                        <packagedElement xmi:type="uml:Class" xmi:id="EAID_5A9CB912_F283_41ae_9E4D_D73598C576AE" name="ExternalReference" visibility="public" isAbstract="true">
                        </packagedElement>
                        <element xmi:idref="EAID_5A9CB912_F283_41ae_9E4D_D73598C576AE" xmi:type="uml:Class" name="ExternalReference" scope="public">
                            <model package="EAPK_EA154075_33FD_455d_B07A_FCBD08A7882D" tpos="0" ea_localid="664" ea_eleType="element"/>
                            <properties isSpecification="false" sType="Class" nType="0" scope="public" isRoot="false" isLeaf="false" isAbstract="true" isActive="false"/>
                            <project author="atkinp" version="1.0" phase="1.0" created="2019-07-24 08:42:52" modified="2020-06-23 19:23:20" complexity="1" status="Proposed"/>
                            <code gentype="Java"/>
                            <style appearance="BackColor=-1;BorderColor=-1;BorderWidth=-1;FontColor=-1;VSwimLanes=1;HSwimLanes=1;BorderStyle=0;"/>
                            <tags/>
                            <xrefs/>
                            <extendedProperties tagged="0" package_name="Common"/>
                        </element>
                    </xmi:XMI>""")
        self.package = UMLPackage("id", "name")

    def test_class_parse(self):
        element = self.tree.find("packagedElement")
        self.assertIsNotNone(element)

        clazz = class_parse(self.package, element, self.tree)
        self.assertIsNotNone(clazz)
        self.assertEqual(clazz.is_abstract, True)
        self.assertEqual(clazz.status, "Proposed")


class TestXMIAssociationParse(unittest.TestCase):
    def setUp(self):
        self.tree = etree.fromstring("""
                    <xmi:XMI xmi:version="2.1" xmlns:uml="http://schema.omg.org/spec/UML/2.1" xmlns:xmi="http://schema.omg.org/spec/XMI/2.1" xmlns:thecustomprofile="http://www.sparxsystems.com/profiles/thecustomprofile/1.0">
                        <packagedElement xmi:type="uml:Association" xmi:id="EAID_3CFEE303_0F7B_46c6_81DE_D14F1BED8EA7" visibility="public">
                            <memberEnd xmi:idref="EAID_dstFEE303_0F7B_46c6_81DE_D14F1BED8EA7"/>
                            <memberEnd xmi:idref="EAID_srcFEE303_0F7B_46c6_81DE_D14F1BED8EA7"/>
                            <ownedEnd xmi:type="uml:Property" xmi:id="EAID_srcFEE303_0F7B_46c6_81DE_D14F1BED8EA7" visibility="public" association="EAID_3CFEE303_0F7B_46c6_81DE_D14F1BED8EA7" isStatic="false" isReadOnly="false" isDerived="false" isOrdered="false" isUnique="true" isDerivedUnion="false" aggregation="composite">
                                <type xmi:idref="EAID_DEFD1F62_622E_479b_8CB9_E219E818F917"/>
                                <lowerValue xmi:type="uml:LiteralInteger" xmi:id="EAID_LI000003__0F7B_46c6_81DE_D14F1BED8EA7" value="0"/>
                                <upperValue xmi:type="uml:LiteralUnlimitedNatural" xmi:id="EAID_LI000004__0F7B_46c6_81DE_D14F1BED8EA7" value="-1"/>
                            </ownedEnd>
                        </packagedElement>
                    </xmi:XMI>""")
        self.package = UMLPackage("id", "name")
        self.source = UMLClass(self.package, "source", "id")
        self.dest = UMLClass(self.package, "dest", "id")

    def test_association_parse(self):
        element = self.tree.find("packagedElement")
        self.assertIsNotNone(element)

        source_element = element.find("ownedEnd")
        dest_element = element.find("ownedEnd")
        assocation = association_parse(self.package, source_element, dest_element, self.source, self.dest)

        self.assertEqual(assocation.cardinality, Cardinality.MANY_TO_MANY)
        self.assertEqual(assocation.association_type, UMLAssociationType.COMPOSITION)



#!/usr/bin/python
# This file is used as an entry point so requires mdg package to be installed into site-packages
# So after a pip or setup.py install you can just cd to the recipie folder and call mdg_generate

# import sys
import os
import logging


logger = logging.getLogger('mdg')
logger.propagate = False
logger.setLevel(logging.INFO)
ch = logging.StreamHandler()
formatter = logging.Formatter('%(asctime)s | %(name)s | %(levelname)s | %(message)s')
ch.setFormatter(formatter)
logger.addHandler(ch)


def generate(args):
    os.environ.setdefault("PYMDG_SETTINGS_MODULE", args.recipie_path)
    from ..generate import generate
    generate()


def validate(args):
    os.environ.setdefault("PYMDG_SETTINGS_MODULE", args.recipie_path)
    from ..validate import validate
    validate()


def startproject(args):
    print('Not Implemented Yet')
    print('Val:((%s))' % args)


def main():
    import argparse

    parser = argparse.ArgumentParser(description='Model Driven Generation Engine')
    subparsers = parser.add_subparsers(title='subcommands', description='valid subcommands', help='subcommand help')

    parser_a = subparsers.add_parser('generate', help='Generate files from a model using a recipie')
    parser_a.add_argument('recipie_path', type=str, help='The path to the recipie config file')
    parser_a.set_defaults(func=generate)

    parser_b = subparsers.add_parser('validate', help='Validate files for a model using a recipie')
    parser_b.add_argument('recipie_path', type=str, help='The path to the recipie config file')
    parser_b.set_defaults(func=validate)

    parser_c = subparsers.add_parser('startproject', help='Create project with recipie and templates')
    parser_c.add_argument('project_type', choices=['django', 'schema', 'java'], help='The type of project')
    parser_c.add_argument('project_path', type=str, help='The path to the project')
    parser_c.set_defaults(func=startproject)

    args = parser.parse_args()
    try:
        func = args.func
    except AttributeError:  # (https://bugs.python.org/issue16308)
        parser.error("too few arguments")
    func(args)


if __name__ == '__main__':
    main()



import requests
import json
from base64 import b64encode
import os
import logging


def bearer_auth(api_client_id, api_client_secret):
    data = """grant_type=client_credentials&client_id={}&client_secret={}&resource=http://GE-SemanticHub-Explorer""".format(api_client_id, api_client_secret)

    # Create a new Service charge transaction by calling API
    headers = {'Content-type': 'application/x-www-form-urlencoded'}
    response = requests.post('https://login.microsoftonline.com/db3d428f-0f97-449a-b683-4df57b0a73c8/oauth2/token', data=data, headers=headers)

    json_response = json.loads(response.content)

    if 'access_token' in json_response.keys():
        return "Bearer {}".format(json_response['access_token'])
    else:
        return None


def basic_auth(user, token):
    b64 = b64encode("{}:{}".format(user, token).encode("ascii"))
    return 'Basic {}'.format(b64.decode("ascii"))


def http_get_json(url, auth, session=None):
    """
    A wrapper around the requests get function which will call SH and load the json response.
    """
    if type(url) is dict:
        logging.info("QueueTriggerPattern request called to get url but url is already dict. Assuming loaded json so returning input.")
        return url
    if url[:5] == "http:":
        url = "https:" + url[5:]

    headers = {
        'Content-type': 'application/json',
        'Authorization': auth,
    }

    if session is None:
        session = requests.Session()

    response = session.get(url, headers=headers, verify=False)
    return json.loads(response.text), session


def http_post(url, auth, data=None, file=None, session=None):

    # Create a new Service charge transaction by calling API
    headers = {
        'authorization': auth,
        'X-Atlassian-Token': 'nocheck',
    }

    media_types = {'png': 'image/png'}

    if session is None:
        session = requests.Session()

    if file is not None:
        with open(file, 'rb') as fp:
            basename = os.path.basename(file)
            files = {'file': (basename, fp, media_types[basename.split('.')[-1]])}
            response = session.post(url, files=files, headers=headers, verify=False)
    else:
        headers['Content-type'] = 'application/json'
        response = session.post(url, data=data, headers=headers, verify=False)

    if response.status_code not in (200, 201):
        raise Exception("POST {} Error: ({}) {} \nHeaders: {}".format(url, response.status_code, response.content, response.headers))

    return response


def http_put(url, auth, data=None, file=None, session=None):

    # Create a new Service charge transaction by calling API
    headers = {
        'authorization': auth,
        'X-Atlassian-Token': 'nocheck',
    }

    media_types = {'png': 'image/png'}

    if session is None:
        session = requests.Session()

    if file is not None:
        with open(file, 'rb') as fp:
            basename = os.path.basename(file)
            files = {'file': (basename, fp, media_types[basename.split('.')[-1]])}
            response = session.put(url, files=files, headers=headers, verify=False)
    else:
        headers['Content-type'] = 'application/json'
        response = session.put(url, data=data, headers=headers, verify=False)

    if response.status_code not in (200, 201):
        raise Exception("PUT {} Error: ({}) {} \nHeaders: {}".format(url, response.status_code, response.content, response.headers))

    return response



import unittest

from mdg.tools.case import camelcase, snakecase


class TestUMLModel(unittest.TestCase):
    def setUp(self):
        pass

    def test_camel(self):
        self.assertEqual("TestCase", camelcase("TestCase"))
        self.assertEqual("TestCase", camelcase("testCase"))
        self.assertEqual("TestCase", camelcase("test case"))
        self.assertEqual("TestCase", camelcase("Test Case"))
        self.assertEqual("TestCase", camelcase("Test_Case"))

    def test_snake(self):
        self.assertEqual("test_case", snakecase("testCase"))
        self.assertEqual("test_case", snakecase("test_case"))
        self.assertEqual("test_case", snakecase("TestCase"))
        self.assertEqual("test_case", snakecase("test case"))
        self.assertEqual("test_case", snakecase("Test Case"))
        self.assertEqual(None, snakecase(None))
        self.assertEqual("eftpos", snakecase("EFTPOS"))



import unittest

from mdg.uml import UMLClass, UMLPackage, UMLAssociation, Cardinality, UMLAssociationType


class TestUMLModel(unittest.TestCase):
    def setUp(self):
        self.root_package = UMLPackage("1", "root")
        child = UMLPackage("2", "child1", self.root_package)
        self.root_package.children.append(child)
        cls = UMLClass(child, "class1", "3")
        child.classes.append(cls)

    def test_find_package(self):
        res = self.root_package.find_by_id("2")
        self.assertEqual(UMLPackage, type(res))
        self.assertEqual("child1", res.name)

    def test_find_class(self):
        res = self.root_package.find_by_id("3")
        self.assertEqual(UMLClass, type(res))
        self.assertEqual("class1", res.name)

    def test_string_to_multiplicity(self):
        assoc = UMLAssociation(self.root_package, self.root_package.children[0].classes[0], self.root_package.children[0].classes[0], 1)
        self.assertEqual(assoc.string_to_multiplicity("0..1"), ("0", "1"))
        self.assertEqual(assoc.string_to_multiplicity("0..*"), ("0", "*"))
        self.assertEqual(assoc.string_to_multiplicity("*..1"), ("*", "1"))

    def test_association_cardinality(self):
        assoc = UMLAssociation(self.root_package, self.root_package.children[0].classes[0], self.root_package.children[0].classes[0], 1)

        assoc.source_multiplicity = ("0", "1")
        assoc.destination_multiplicity = ("0", "*")
        self.assertEqual(assoc.cardinality, Cardinality.ONE_TO_MANY)

        assoc.source_multiplicity = ("0", "1")
        assoc.destination_multiplicity = ("0", "1")
        self.assertEqual(assoc.cardinality, Cardinality.ONE_TO_ONE)

        assoc.source_multiplicity = ("0", "*")
        assoc.destination_multiplicity = ("0", "1")
        self.assertEqual(assoc.cardinality, Cardinality.MANY_TO_ONE)

        assoc.source_multiplicity = ("0", "*")
        assoc.destination_multiplicity = ("0", "*")
        self.assertEqual(assoc.cardinality, Cardinality.MANY_TO_MANY)

    def test_association_type(self):
        assoc = UMLAssociation(self.root_package, self.root_package.children[0].classes[0], self.root_package.children[0].classes[0], 1)
        self.assertEqual(assoc.association_type, UMLAssociationType.ASSOCIATION)
        assoc = UMLAssociation(self.root_package, self.root_package.children[0].classes[0], self.root_package.children[0].classes[0], 1, UMLAssociationType.COMPOSITION)
        self.assertEqual(assoc.association_type, UMLAssociationType.COMPOSITION)



#!/usr/bin/python
import json
import os
import logging
from typing import Dict, Optional, List, Any

from jinja2 import Environment, FileSystemLoader, Template, BaseLoader


from .config import settings
from .tools.filters import get_filters

from .uml import UMLPackage, UMLInstance


logger = logging.getLogger(__name__)


def output_level_copy(source_filename: str, dest_file_template: Template, package: UMLPackage) -> None:
    """ Render a jinja template as pass a UML package as data
    """

    # Render template for UML Package
    dest_filename: str = os.path.abspath(dest_file_template.render(package=package))
    dirname: str = os.path.dirname(dest_filename)

    # make sure computed distination path exists
    if not os.path.exists(dirname):
        os.makedirs(dirname)

    with open(source_filename) as source_fh:
        with open(dest_filename, 'w') as dest_fh:
            dest_fh.write(source_fh.read())

    logger.debug(f"Created {dest_filename}")


def output_level_package(source_template: Template, dest_file_template: Template, package: UMLPackage) -> None:
    """ Render a jinja template as pass a UML package as data
    """

    # Render template for UML Package
    dest_filename: str = os.path.abspath(dest_file_template.render(package=package))
    dirname: str = os.path.dirname(dest_filename)

    # make sure computed distination path exists
    if not os.path.exists(dirname):
        os.makedirs(dirname)

    with open(dest_filename, 'w') as fh:
        fh.write(source_template.render(package=package))

    logger.debug(f"Created {dest_filename}")


def output_level_enum(source_template: Template, dest_file_template: Template, filter_template: Optional[Template], package: UMLPackage) -> None:
    """ Render a jinja template for each enumeration in the supplied package
    """

    # Loop through all enumerations in the UML Package, cheeck the filter result and output if True
    for enum in package.enumerations:
        if filter_template is None or filter_template.render(enum=enum) == "True":
            dest_filename = os.path.abspath(dest_file_template.render(enum=enum))
            dirname = os.path.dirname(dest_filename)

            # make sure computed distination path exists
            if not os.path.exists(dirname):
                os.makedirs(dirname)

            with open(dest_filename, 'w') as fh:
                fh.write(source_template.render(enum=enum))

            logger.debug(f"Created {dest_filename}")


def output_level_class(source_template: Template, dest_file_template: Template, filter_template: Optional[Template], package: UMLPackage) -> None:
    """ Render a jinja template for each class in the supplied package
    """

    # Loop through all classes in the UML Package, cheeck the filter result and output if True
    for cls in package.classes:
        if filter_template is None or filter_template.render(cls=cls) == "True":
            dest_filename = os.path.abspath(dest_file_template.render(cls=cls))
            dirname = os.path.dirname(dest_filename)

            # make sure computed distination path exists
            if not os.path.exists(dirname):
                os.makedirs(dirname)

            with open(dest_filename, 'w') as fh:
                fh.write(source_template.render(cls=cls))

            logger.debug(f"Created {dest_filename}")


def output_level_assoc(source_template: Template, dest_file_template: Template, filter_template: Optional[Template], package: UMLPackage):
    """ Render a jinja template for each association in the supplied package
    """

    # Loop through all associations in the UML Package, cheeck the filter result and output if True
    for assoc in package.associations:
        if filter_template is None or filter_template.render(association=assoc) == "True":
            dest_filename = os.path.abspath(dest_file_template.render(association=assoc))
            dirname = os.path.dirname(dest_filename)

            # make sure computed distination path exists
            if not os.path.exists(dirname):
                os.makedirs(dirname)

            with open(dest_filename, 'w') as fh:
                fh.write(source_template.render(association=assoc))

            logger.debug(f"Created {dest_filename}")


def output_model(package: UMLPackage) -> None:
    """ Loops through model templates in the settings and calls render functions

        Each template consists of:
            dest: The filename of the output
            level: Do we generate a file for each package/class/enumeration/association or root
            source: Path to the jinja2 template
            filter: If supplied, If supplied The template must output "True" for a file to be generated. E.g. "{% if package.classes %}True{% else %}False{% endif %}"
    """
    logger.info("Generating model output for package {}".format(package.path))

    # Create jinja2 filter dict to pass into templates
    filters = get_filters()

    # Create jinja2 environmeent with filters
    import os
    default_templates = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'templates')
    source_env = Environment(loader=FileSystemLoader([settings['templates_folder'], default_templates]))
    source_env.filters = {**source_env.filters, **filters}
    dest_env = Environment(loader=BaseLoader())
    dest_env.filters = {**source_env.filters, **filters}

    # Loop through all template definitions in the config file
    template_definition: Dict
    for template_definition in settings['model_templates']:
        dest_file_template: Template = dest_env.from_string(os.path.join(settings['dest_root'], template_definition['dest']))

        if template_definition['level'] == 'copy':
            if package.parent is None:
                source_file = os.path.join(settings['templates_folder'], template_definition['source'])
                if not os.path.isfile(source_file):
                    source_file = os.path.join(default_templates, template_definition['source'])
                output_level_copy(source_file, dest_file_template, package)
        else:
            # Create jinja2 teemplates for the source file and dest file name
            source_template: Template = source_env.get_template(template_definition['source'])

            # Filter template is optional and used to skip a file generation.
            filter_template: Optional[Template] = None
            if 'filter' in template_definition.keys():
                filter_template = Template(template_definition['filter'])

            # Select the output renderer based on the level requested
            if template_definition['level'] == 'package':
                if filter_template is None or filter_template.render(package=package) == "True":
                    output_level_package(source_template, dest_file_template, package)
            elif template_definition['level'] == 'class':
                output_level_class(source_template, dest_file_template, filter_template, package)
            elif template_definition['level'] == 'enumeration':
                output_level_enum(source_template, dest_file_template, filter_template, package)
            elif template_definition['level'] == 'assocication':
                output_level_assoc(source_template, dest_file_template, filter_template, package)
            elif template_definition['level'] == 'root':
                if package.parent is None:
                    output_level_package(source_template, dest_file_template, package)
            else:
                raise ValueError("'{}' is not a valid template level".format(template_definition['level']))

    # Walk through the package hierarchy and recurse output
    child: UMLPackage
    for child in package.children:
        output_model(child)


def output_test_cases(test_cases: List[UMLInstance]) -> None:
    """ Test cases are parse into a list of UML instances. Loop through list and serialise
    """
    if settings['test_templates'] is None:
        logger.warn("No test templates")
        return

    logger.info("Generating test case output")

    for case in test_cases:
        serialised = json.dumps(serialize_instance(case), indent=2)

        template_definition: Dict
        for template_definition in settings['test_templates']:
            filename_template: Template = Template(template_definition['dest'])
            filename: str = os.path.abspath(filename_template.render(ins=case))
            dirname: str = os.path.dirname(filename)

            # make sure computed distination path exists
            if not os.path.exists(dirname):
                os.makedirs(dirname)

            with open(filename, 'w') as fh:
                fh.write(serialised)


def serialize_instance(instance: UMLInstance):
    """ Generates a dictionary of attributes, values, dicts and lists from UML instance
        Recurses through associations originaing from supplied instance to serialise sub-objects
    """
    ret: Dict = {}

    # Extract attributes and their values
    for attr in instance.attributes:
        ret[attr.name] = attr.value

    # Loop through associations originating from this instance
    for assoc in instance.associations_from:
        dest: Any = assoc.destination
        # If the multiplicity is multiple then generate list
        if assoc.destination_multiplicity[1] == '*':
            if assoc.destination.name not in ret.keys():
                ret[assoc.destination.name] = [serialize_instance(dest), ]
            else:
                ret[assoc.destination.name].append(serialize_instance(dest))
        # If multiplicity is singular then generate dict
        else:
            ret[assoc.destination.name] = serialize_instance(dest)

    return ret



import os
import glob

from mdg.confluence.util import basic_auth, http_post, http_get_json

ORG = "my org"


def update_images(auth, content_id, path):
    attachments, session = http_get_json("https://{}.atlassian.net/wiki/rest/api/content/{}/child/attachment".format(ORG, content_id), auth)

    files = [f for f in glob.glob(path + "/*.png", recursive=False)]

    for file in files:
        basename = os.path.basename(file)
        conf_attachment = None
        for attachment in attachments['results']:
            if attachment['title'] == basename:
                conf_attachment = attachment

        # Is the file already an attachment in confluence
        if conf_attachment is not None:
            # Update if different file size
            if os.path.getsize(file) != conf_attachment['extensions']['fileSize']:
                url = "https://{}.atlassian.net/wiki/rest/api/content/{}/child/attachment/{}/data".format(ORG, content_id, attachment['id'])
                http_post(url, auth, None, file, None)
                print("Uploaded local file {} to confluence".format(file))
            else:
                print("Skipping local file {} as same file size detected in confluence".format(file))
        else:
            # TODO: Upload new attachment
            url = "https://{}.atlassian.net/wiki/rest/api/content/{}/child/attachment/".format(ORG, content_id)
            http_post(url, auth, None, file, None)
            print("Uploaded new local file {} to confluence".format(file))


if __name__ == '__main__':
    import argparse

    parser = argparse.ArgumentParser()
    parser.add_argument('user', help='confluence username')
    parser.add_argument('token', help='confluence API token')
    parser.add_argument('page', help='confluence page id')
    parser.add_argument('path', help='path to images foler')
    args = parser.parse_args()

    auth = basic_auth(args.user, args.token)
    update_images(auth, args.page, args.path)



from ..config import settings
from .case import camelcase, snakecase, titlecase, sentencecase


def get_filters():
    # Create jinja2 filter dict to pass into templates
    filters = {
        'camelcase': camelcase,
        'snakecase': snakecase,
        'titlecase': titlecase,
        'sentencecase': sentencecase,
    }

    if settings['case_package'] == "CamelCase":
        filters['case_package'] = camelcase
    elif settings['case_package'] == "snake_case":
        filters['case_package'] = snakecase
    else:
        filters['case_package'] = camelcase

    if settings['case_class'] == "CamelCase":
        filters['case_class'] = camelcase
    elif settings['case_class'] == "snake_case":
        filters['case_class'] = snakecase
    else:
        filters['case_class'] = camelcase

    if settings['case_attribute'] == "CamelCase":
        filters['case_attribute'] = camelcase
    elif settings['case_attribute'] == "snake_case":
        filters['case_attribute'] = snakecase
    else:
        filters['case_class'] = snakecase

    return filters



import unittest

from mdg.parse import parse
from mdg.validate import validate_package
from mdg.render import output_model


class TestSample_DrawIO_Django(unittest.TestCase):
    def setUp(self):
        # Env must be set before tests
        # self.recipie_path = './sample_recipie' + "/config-test.yaml"
        # os.environ.setdefault("PYMDG_SETTINGS_MODULE", self.recipie_path)

        self.model_package, self.test_cases = parse()

    def test_parse(self):
        self.assertEqual("SampleIndustry", self.model_package.name)

    def test_validate(self):
        # Deliberate error in test config
        errors = validate_package(self.model_package)
        self.assertEquals(len(errors), 1)

    def test_render_model(self):
        self.assertGreater(len(self.model_package.children), 0)
        # Test config has built in failure no child packages will be rendered
        self.assertRaises(ValueError, output_model, self.model_package)

        # Manually go to child package
        self.assertRaises(ValueError, output_model, self.model_package.children[0])



import os
import setuptools


def find_packages(srcdir):
    package_list = []
    badnames = ["__pycache__", "venv", ]
    for root, dirs, files in os.walk(srcdir):
        if not any(bad in root for bad in badnames):
            if "__init__.py" in files:
                package_list.append(root.replace("/", ".").replace("\\", ".").strip('.'))
    return package_list


pymdg_packages = find_packages('./mdg')

with open("README.md", "r") as fh:
    long_description = fh.read()


setuptools.setup(
    name='pymdg',
    version='0.4a4',
    author='Semprini',
    author_email='dont@contact.me',
    description='Model driven genration - from UML to Code & Docs',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/semprini/pyMDG',
    packages=["mdg", ] + pymdg_packages,
    include_package_data=True,
    entry_points={
        'console_scripts': [
            'mdg-tool=mdg.tools.mdg_tool:main',
        ]
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License (GPL)",
        "Operating System :: OS Independent",
    ],
    install_requires=[
        "lxml",
        "jinja2",
        "pyyaml",
    ],
    python_requires='>=3.7',
)



#!/usr/bin/python
from __future__ import annotations
from typing import List, Union, Optional, Any, Tuple
from enum import Enum

from mdg.config import settings


class UMLStatuses(Enum):
    Proposed = 1
    Approved = 2
    Implemented = 3
    Mandatory = 4
    Validated = 5


class UMLPackage(object):
    def __init__(self, id: Union[int, str], name: str, parent: Optional[UMLPackage] = None, stereotype: Optional[str] = None):
        self.classes: List[UMLClass] = []
        self.associations: List[UMLAssociation] = []
        self.children: List[UMLPackage] = []
        self.instances: List[UMLInstance] = []
        self.enumerations: List[UMLEnumeration] = []
        self.parent: Optional[UMLPackage] = parent
        self.stereotype: Optional[str] = stereotype
        self.inherited_stereotypes: List[Tuple[str, UMLPackage]] = []
        self.name: str = name
        self.id: Union[int, str] = id
        self.diagrams: Any = []
        self.documentation: str = ""
        self.element: Any = None
        self.root_element: Any = None
        self.status: Optional[UMLStatuses] = None

        if self.parent is None:
            self.root_package: UMLPackage = self
            self.path = '/' + settings['root_package'] + '/'
        else:
            self.root_package = self.parent.root_package
            self.inherited_stereotypes += self.parent.inherited_stereotypes
            self.path = self.parent.path + self.name + '/'

    def __str__(self):
        return f"{self.name}"

    def find_by_id(self, id: Union[int, str]):
        """ Finds UMLPackage, UMLClass, UMLEnumeration or UMLInstance object with specified Id
        Looks for classes part of this package and all sub-packages
        """
        if self.id == id:
            return self

        for cls in self.classes:
            if cls.id == id:
                return cls

        for ins in self.instances:
            if ins.id == id:
                return ins

        for enum in self.enumerations:
            if enum.id == id:
                return enum

        for child in self.children:
            res = child.find_by_id(id)
            if res is not None:
                return res

    def add_child(self, id: Union[int, str], name: str, stereotype=None) -> UMLPackage:
        package: UMLPackage = UMLPackage(id, name, self, stereotype)
        self.children.append(package)
        return package


class UMLInstance(object):
    def __init__(self, package: UMLPackage, name: str, id: Union[int, str]):
        self.attributes: List[UMLAttribute] = []
        self.associations_from: List[UMLAssociation] = []
        self.associations_to: List[UMLAssociation] = []
        self.package: UMLPackage = package
        self.stereotype: Optional[str] = None
        self.name: str = name
        self.id: Union[int, str] = id
        self.documentation: str = ""
        self.status: Optional[UMLStatuses] = None

    def __str__(self) -> str:
        return f"{self.name}"


class UMLAssociationType(Enum):
    ASSOCIATION = 1
    COMPOSITION = 2
    AGGREGATION = 3


class Cardinality(Enum):
    MANY_TO_ONE = 1
    ONE_TO_MANY = 2
    MANY_TO_MANY = 3
    ONE_TO_ONE = 4


class UMLAssociation(object):
    def __init__(self, package: UMLPackage, source: Union[UMLClass, UMLInstance], destination: Union[UMLClass, UMLInstance], id: Union[int, str], assoc_type=UMLAssociationType.ASSOCIATION):
        self.package: UMLPackage = package

        self.association_type: UMLAssociationType = assoc_type
        self.source: Union[UMLClass, UMLInstance] = source
        self.source_name: Optional[str] = None
        self.source_multiplicity: Tuple[str, str] = ('0', '0')
        source.associations_from.append(self)

        self.destination: Union[UMLClass, UMLInstance] = destination
        self.destination_multiplicity: Tuple[str, str] = ('0', '0')
        self.destination_name: Optional[str] = None
        destination.associations_to.append(self)
        self.documentation: str = ""
        self.id: Union[int, str] = id

    def __str__(self) -> str:
        return f"{self.source_name}({self.source_multiplicity}) -> {self.destination_name}({self.destination_multiplicity})"

    @property
    def cardinality(self):
        cardinality = None
        # Use multiplicities to calculate the CARDINALITY of THYE association
        if self.source_multiplicity[1] == '*' and self.destination_multiplicity[1] in ('0', '1'):
            cardinality = Cardinality.MANY_TO_ONE
        elif self.destination_multiplicity[1] == '*' and self.source_multiplicity[1] in ('0', '1'):
            cardinality = Cardinality.ONE_TO_MANY
        elif self.destination_multiplicity[1] == '*' and self.source_multiplicity[1] == '*':
            cardinality = Cardinality.MANY_TO_MANY
        elif self.destination_multiplicity[1] in ('0', '1') and self.source_multiplicity[1] in ('0', '1'):
            cardinality = Cardinality.ONE_TO_ONE

        return cardinality

    def string_to_multiplicity(self, value):
        output = ('', '')
        if value == "0..*":
            output = ("0", "*")
        elif value == "1..*":
            output = ("1", "*")
        elif value in ("1", "0..1"):
            output = ("0", "1")
        elif value == "1..1":
            output = ("1", "1")
        elif value == "*..1":
            output = ("*", "1")
        elif value == "*..0":
            output = ("*", "0")

        return output


class UMLEnumeration(object):
    def __init__(self, package: UMLPackage, name: str, id: Union[int, str]):
        self.values: List[str] = []
        self.package: UMLPackage = package
        self.name: str = name
        self.id: Union[int, str] = id
        self.documentation: str = ""
        self.status: Optional[UMLStatuses] = None

    def __str__(self) -> str:
        return f"{self.name}"


class UMLClass(object):
    def __init__(self, package: UMLPackage, name: str, id: Union[int, str]):
        self.name: str = name
        self.id: Union[int, str] = id
        self.attributes: List[UMLAttribute] = []
        self.associations_from: List[UMLAssociation] = []
        self.associations_to: List[UMLAssociation] = []
        self.package: UMLPackage = package
        self.generalization: Optional[UMLClass] = None
        self.generalization_id: Union[None, int, str] = None
        self.specialized_by: List[UMLClass] = []
        self.stereotypes: List[str] = []
        self.id_attribute: Optional[UMLAttribute] = None
        self.documentation: str = ""
        self.is_abstract: bool = False
        self.status: Optional[UMLStatuses] = None

        for inherited_stereotype, inherited_package in package.inherited_stereotypes:
            if not hasattr(self, inherited_stereotype):
                setattr(self, inherited_stereotype, inherited_package)

    def __str__(self) -> str:
        return f"{self.name}"


class UMLAttribute(object):
    def __init__(self, parent: UMLClass, name: str, id: Union[int, str]):
        self.parent: UMLClass = parent
        self.name: str = name
        self.id: Union[int, str] = id
        self.is_unique: bool = False
        self.stereotype: Optional[str] = None
        self.classification: Optional[UMLClass] = None
        self.classification_id: Union[None, int, str] = None
        self.documentation: str = ""
        self.type: Optional[str] = None
        self.dest_type: Optional[str] = None
        self.value: Optional[str] = None
        self.visibility: bool = True
        self.is_id: bool = False
        self.length: int = 0
        self.validations: List[str] = []

    def __str__(self) -> str:
        return f"{self.name}"



import unittest

from mdg.uml import UMLAttribute, UMLInstance, UMLPackage
from mdg.render import serialize_instance


class TestJSONOutput(unittest.TestCase):
    def setUp(self):
        self.package = UMLPackage("id", "name")
        self.instance = UMLInstance(self.package, "test1", 1)
        attr = UMLAttribute(self.package, 'testatt', 1)
        attr.value = "testval"
        self.instance.attributes.append(attr)
        self.package.instances.append(self.instance)

    def test_object_serialise(self):
        data = serialize_instance(self.instance)
        self.assertEqual(data['testatt'], 'testval')
